//	Header for Lattice dBC III Library

#ifndef _DEFD_DBC_H_
#define _DEFD_DBC_H_ 1


#ifndef LINT_ARGS
#define LINT_ARGS	1
#endif
#ifndef MSC
#define MSC			//  Microsoft C Option
#endif
#define dBC3			//  dBC III Option

				//  --------- dBC III Data Types ---------
#define BYTE	unsigned char	//  unsigned 1 byte data
#define U2BYTES unsigned short	//  unsigned 2 bytes data
#define RECNUM	long		//  record number
#define FLDWIDTH unsigned char	//  field width in bytes or characters
#define FLDDEC	unsigned char	//  decimal places in field
#define FLDNUM	unsigned char	//  number of fields per record
#define RECLEN	unsigned short	//  record length (unsigned int ??)
#define KEYLEN	unsigned char	//  key length
#define BLKNO	long		//  block number type
#define NBYTES	long

				//  ------- dBase III restrictions -------
#define dBC_MAX_RECLEN		4000	//  maximum record length
#define dBC_MAX_FIELDS		128	//  maximum number of fields
#define dBC_MAX_CHAR_FLD	254	//  maximum length of char fields


//	.DBF record field definition structure

#pragma pack(2)
struct	dbc_DBF_field_layout
{
    char	fieldnm [11] ;	//  field name terminating with EOS
    char	type  ; 	//  type of data: C, N, D, L, or M
    FLDWIDTH	width ; 	//  field width
    FLDDEC	dec   ; 	//  length of decimal places
};
#define dBFIELD 		struct dbc_DBF_field_layout


//	.DBF file definition structure (record status)

struct	dbc_DBF_file_status
{
	int	_dbffn	 ;	//  file number
	BYTE	_dbfmode ;	//  mode of open file
	RECNUM	_alcnum  ;	//  number of records allocated
	int	_dbfupd  ;	//  data file update indicator
	RECLEN	_len	 ;	//  record length
				//  *** I/O buffer information
	U2BYTES _bufsize ;	//  I/O buffer size
	BYTE	*_bfptr  ;	//  I/O buffer location pointer
	U2BYTES _maxnum  ;	//  maximum number of records in I/O buffer
	RECNUM	_first	 ;	//  number of first record in buffer
	U2BYTES _bfhas	 ;	//  number of records currently contained in buffer
	BYTE	_month	 ;	//  file update date: month
	BYTE	_day	 ;	//		      day
	BYTE	_year	 ;	//		      year
	FLDNUM	_nflds	 ;	//  number of fields per record
	dBFIELD *_flds	 ;	//  pointer to array of dBC III field structure (dBFIELD)
	U2BYTES _begdata ;	//  beginning-of-data offset
};
#define DBFFILE 		struct dbc_DBF_file_status


//	.NDX file definition structure (record status)

struct	dbc_NDX_file_status
{
	int	_ndxfn	 ;	//  file number
	BYTE	_ndxmode ;	//  mode of open file
	BLKNO	_orootbk ;	//  original root block number
	BLKNO	_oeofbk  ;	//  original end-of-file block number
	BLKNO	_rootblk ;	//  current root block number
	BLKNO	_eofblk  ;	//  current end-of-file block number
	KEYLEN	_keylen  ;	//  key length
	KEYLEN	_kptrlen ;	//  length of key and pointer (_keylen + ptr len)
	BYTE	_maxknum ;	//  maximum number of keys in a block
	BYTE	_keytype ;	//  key type: 'Character', 'Numeric' or 'Date'
	struct	_seqacc 	//  *** used for sequential accessing
	{
		BLKNO	_nxblkno  ;	//  block number
		BYTE	_nxidno   ;	//  index id number
		BYTE	*_nxbfptr ;	//  .NDX I/O buffer pointer
		BYTE	_nxusage  ;	//  buffer usage indicator
	}	_svector[32],		//  sequential access vector
		*_svptr, *_leafsv, *_endofsv ;
					//  pointers to sequential access vector
};
#define NDXFILE 		struct dbc_NDX_file_status


//	.DBT file definition structure (record status)

struct	dbc_DBT_file_status
{
	int	_dbtfn	 ;	//  file number
	BYTE	_dbtmode ;	//  mode of open file
	BLKNO	_odbteof ;	//  original end-of-file block number
	BLKNO	_dbteof  ;	//  current end-of-file block number
};
#define DBTFILE 		struct dbc_DBT_file_status
#pragma pack()


				//  ------ record status definition ------
#define ACTIVE		0	//  record is active
#define INACTIVE	1	//  record is inactive (marked deleted)

				//  -------- key type definitions --------
#define CHARKEY 	'C'     //  CHARACTER key
#define NDKEY		'N'     //  NUMERIC or DATE key


//	Return values from dBC functions.  See "dbc_doc.h" for details.

#define SUCCESS 	0
#define d_ERROR 	1
#define d_NODBF 	2
#define d_BADREC	3
#define d_WTFAIL	4
#define d_NONDX 	5
#define d_KYLONG	6
#define d_NOKEY 	7
#define d_MAYBE 	8
#define d_ENDKEY	9
#define d_NOSYNC	10
#define d_TOPKEY	11


//	Function Definitions for Lattice dBC III Library

int	dBakey(char *,char *,long);
int	dBatofld(char *,int,int,char *);
int	dBatokey(char *,char *);
int	dBckey(char *,char *,long *);
int	dBclose(char *);
int	dBcreat(char *,unsigned char,struct dbc_DBF_field_layout *);
int	dBdelete(char *,long);
int	dBfldtoa(char *,int,char *);
int	dBflush(char *);
int	dBfwd(char *);
int	dBgetf(char *,unsigned short *,unsigned char *,unsigned char *,unsigned char *,unsigned char *,struct dbc_DBF_field_layout *);
int	dBgetm(char *,char *,char *);
int	dBgetnr(char *,char *,char *,char *);
int	dBgetpr(char *,char *,char *,char *);
int	dBgetr(char *,long,char *,char *);
int	dBgetrk(char *,char *,char *,char *,char *);
int	dBiclose(char *);
int	dBicreat(char *,char *,unsigned char,char);
int	dBiflsh(char *);
int	dBiopen(char *,char * *);
int	dBkexpr(char *,char *,char *,int *,unsigned char *);
int	dBkeytoa(char *,char *);
int	dBmclose(char *);
int	dBmcreat(char *);
int	dBmopen(char *,char * *);
int	dBnkey(char *,char *,long *);
int	dBopen(char *,char * *);
int	dBpkey(char *,char *,long *);
int	dBputm(char *,char *,char *);
int	dBputr(char *,long,char *);
int	dBputrk(char *,char *,char *,char *);
int	dBrecall(char *,long);
int	dBrewind(char *);
int	dBrmvkey(char *,char *,long);
int	dBrmvr(char *,long);
int	dBsize(char *,long *);
int	db_Size(unsigned char *,long *);
void	dBstrcpy(char *,char,int,char *);
int	dBtkey(char *,char *,long *);
int	dBupdr(char *,long,char *);

extern	int	dbc_error ;
extern	char	dbc_vers[] ;
extern	U2BYTES _dbcmsiz ;
extern	U2BYTES _montbl[] ;	// See dbcgdata.c


#endif	// avoid duplicate definition
