typedef unsigned char  BYTE;	/* unsigned 1 byte data */
typedef unsigned short U2BYTES; /* unsigned 2 bytes data */
typedef long	      RECNUM;	/* record number */
typedef BYTE	      FLDWIDTH; /* field width in bytes or characters */
typedef BYTE	      FLDDEC;	/* decimal places in field */
typedef BYTE	      FLDNUM;	/* number of fields per record */
typedef unsigned int  RECLEN;	/* record length */
typedef BYTE	      KEYLEN;	/* key length */
typedef long BLKNO;
typedef long	  NBYTES;

/******************************************/
/* .DBF record field definition structure */
/******************************************/
typedef struct
{
	char fieldnm[11]; /* field name terminating with NULL(0) */
	char type;	  /* type of data */
			     /* 'C' - character
				'N' - numeric
				'D' - date
				'L' - logical
				'M' - memo    */
	FLDWIDTH width;   /* field width */
	FLDDEC	 dec;	  /* length of decimal places */
} dBFIELD;

/***** record status definition *****/
#define ACTIVE	  0 /* record is active */
#define INACTIVE  1 /* record is inactive (marked deleted) */

/***** key type definitions *****/
#define CHARKEY   'C' /* CHARACTER key */
#define NDKEY	  'N' /* NUMERIC or DATE key */

/***** File/Record Locking Constants Definitions *****/
#define d_SINGLE  0	/* Single User mode */
#define d_SHARED  1	/* Shared mode */

#define d_ULOCK 0	/* Unlock file/record */
#define d_LOCK	1	/* Lock file/record */
#define d_TLOCK 2	/* Test and lock file/record */
#define d_TEST	3	/* Test if file/record is locked */

#define d_DBF	0	/* for data file */
#define d_NDX	1	/* for index file */
#define d_DBT	2	/* for memo file */

#define d_AUTOLK 0	/* automatic locking/unlocking method */
#define d_MANLK  1	/* manual locking/unlocking method */

/* Flag Used To Indicate Minimum Buffer Size Request
	(Used in 'dBopen' and 'dBiopen')                */
#define d_MINBUF 0x80

/********************************************
* return values from dBC III Plus functions *
*********************************************/
#define SUCCESS   0    /** successful function return code **/
#define d_ERROR   1    /** general error code **/
			  /* This error code is returned by a function that
			     failed due to a fundamental reason such as wrong
			     input values or the file to be accessed is not
			     open. */
#define d_NODBF   2    /** accessed file is not organized in the data **/
		       /*  file(.DBF) format */
#define d_BADREC  3    /** specified record number is out of range, **/
		       /*  i.e., larger than the current number of allocated
			   records */
#define d_WTFAIL  4    /** file-write operation failed, e.g., "disk full" **/
#define d_NONDX   5    /** accessed file is not organized in the index **/
		       /*  file(.NDX) format */
#define d_KYLONG  6    /** too long key expression **/
#define d_NOKEY   7    /** index file(.NDX) is empty or requested key is
			   not found **/
#define d_MAYBE   8    /** requested key was not found in the index file, **/
		       /*  but another key that may immediately follow the
			   requested key was located */
#define d_ENDKEY  9    /** no more key exists in the index file **/
		       /*  the sequential-read key pointer is placed
			   at the end of the index file */
#define d_NOSYNC  10   /** key exists in the index file, but its **/
		       /*  corresponding record does not exist in the
			   specified data file */
#define d_TOPKEY  11   /** top of the index file condition **/
		       /*  the sequential-read key pointer reached
			   the top of the index file */
#define d_MEMERR  12   /** memory allocation error **/
#define d_BUFERR  13   /** private buffer exhasted **/
#define d_NOTFND  14   /** object not found, e.g., unsuccessful search */
#define d_INVDAT  15   /** invalid date */
	/* 16 through 19: unused */
#define d_LOCKED  20   /** record or file already locked by the other **/
#define d_SELFLK  21   /** record or file already locked by self **/
#define d_RECLK   22   /** record already locked by self **/
#define d_DBFLK   23   /** .DBF already locked by self **/
#define d_SIZELK  24   /** size bytes already locked by self **/
#define d_NVLOCK  25   /** record or file never locked by self */
#define d_FREE	  26   /** rcord or file is not locked **/
#define d_RELERR  27   /** previous 'current key' was removed by another
			   process **/

/*** special constant for the dBputr function ***/
#define APPEND ((RECNUM) -987)

extern RECNUM _appnded;
extern int _dbcerr;
extern unsigned int _dbcmsiz;
extern char _dbcver[];
extern int _dlock;
extern int _mwidth;
typedef unsigned short ino_t;		/* i-node number (not used on DOS) */

typedef long time_t;			/* time value */
typedef short dev_t;			/* device code */
typedef long off_t;			/* file offset value */

struct stat
{
	dev_t st_dev;
	ino_t st_ino;
	unsigned short st_mode;
	short st_nlink;
	short st_uid;
	short st_gid;
	dev_t st_rdev;
	off_t st_size;
	time_t st_atime;
	time_t st_mtime;
	time_t st_ctime;
};

typedef struct
{
	int	 _dbffn;    /* file number */
	int	 _len;	    /* record length (including 1 for a status byte) */
	long	 _begdata;  /* beginning-of-data offset */
	long	 _lock;     /* lock info.  >0 record;  0 for size;
			       -1 for all and  -2 for unlocked */

	FLDNUM	 _nflds;    /* number of fields per record */
	dBFIELD  *_flds;    /* pointer to array of dBC III field
			       structure(dBFIELD) */
} DBFFILE;


/*****************************************/
/* index file(.NDX) definition structure */
/*****************************************/
typedef struct
{
	int	 _ndxfn;    /* file number */
	BYTE	 _ndxmode;  /* mode of open file */
	BLKNO	 _orootbk;  /* original root block number */
	BLKNO	 _oeofbk;   /* original end-of-file block number */
	BLKNO	 _rootblk;  /* current root block number */
	BLKNO	 _eofblk;   /* current end-of-file block number */
	KEYLEN	 _keylen;   /* key length */
	KEYLEN	 _kptrlen;  /* length of key and pointer (_keylen + ptr len) */
	BYTE	 _maxknum;  /* maximum number of keys in a block */
	char	 _keytype;  /* key type: 'Character', 'Numeric' or 'Date' */
	BYTE	 _ndxlock;  /* see below */

		/*** used for sequential accessing ***/
	char  *_bigbuf;  /* big buffer */
	long   _bigsize; /* big buffer size */
	char   _bigflag; /* indicates how big buffer is used */

	struct _seqacc
	{
		 BLKNO	_nxblkno;   /* block number */
		 BYTE	_nxidno;    /* index id number */
		 char	*_nxbfptr;  /* .NDX I/O buffer pointer */
		 BYTE	_nxusage;   /* buffer usage indicator */
	} _svector[32],  /* sequential access vector */
	  *_svptr, *_leafsv, *_endofsv;
				/* pointers to sequential access vector */
	BYTE	 *_ckeybuf;	    /* buffer for current key */
	RECNUM	 _crecno;	    /* current record number */
} NDXFILE;
