//<t>   Definitions for Disk, Directory, and File Functions

//<c>   Copyright 1980-1998 by Gerry J. Danen; all rights reserved.


#ifndef _DEFD_GD_FILES_H_
#define _DEFD_GD_FILES_H_ 1


#include<fcntl.h>
#include<io.h>
#include<stat.h>
#include"ms_struc.h"


#define         READ_ONLY       0x01    //  Bits used in attribute
#define         HIDDEN          0x02    //  field of directory entry
#define         SYSTEM          0x04
#define         VOL_ENTRY       0x08
#define         DIRECTORY       0x10
#define         ARCHIVE         0x20
#define         FSPEC           struct file_spec
#define         DIRLIST         struct directory_list
#define         SUBDIR          struct subdir_list


                                        //  File information structure
FSPEC                                   //  used by FindFirstFile and
{                                       //  FindNextFile
    UCHAR       fname[13]       ;       //  File name xxxxxxxx.xxx
    UINT32      fsize           ;       //  File size in bytes
    UINT32      fdate_st        ;       //  Date stamp yyyymmdd
    UINT32      ftime_st        ;       //  Time stamp hhmmss00
    UCHAR       fdate_tx[10]    ;       //  Date stamp dd-mmm-yy
    UCHAR       ftime_tx[ 9]    ;       //  Time stamp hh:mmx or hh:mm:ss
    INT16       fattr           ;       //  Attribute  (1 byte)
    INT16       f_at_read_only  ;       //  Attribute bit read-only
    INT16       f_at_hidden     ;       //  Attribute bit hidden
    INT16       f_at_system     ;       //  Attribute bit system
    INT16       f_at_archive    ;       //  Attribute bit archive
    INT16       f_at_vol_label  ;       //  Attribute bit volume label
    INT16       f_at_sub_dir    ;       //  Attribute bit sub-directory
    INT16       f_at_reserved_1 ;       //  Attribute bit reserved
    INT16       f_at_reserved_2 ;       //  Attribute bit reserved
};


DIRLIST
{
    UCHAR       name [ 8+1]     ;       //  name of file
    UCHAR       ext  [ 3+1]     ;       //  extension
    UCHAR       date [11+1]     ;       //  dd-mmm-yy or dd-mmm-yyyy
    UCHAR       time [ 8+1]     ;       //  hh:mmXM or hh:mm:ss
    UCHAR       size [13+1]     ;       //  9,999,999,999
    UCHAR       s_date[ 4]      ;       //  date for sorting, no \0
    UCHAR       s_time[ 3]      ;       //  time for sorting, no \0
    UCHAR       s_size[ 5]      ;       //  size for sorting, no \0
    FLAG        at_read_only    ;       //  attr: read-only
    FLAG        at_archive      ;       //  attr: archive
    FLAG        at_hidden       ;       //  attr: hidden
    FLAG        at_system       ;       //  attr: system
    FLAG        at_sub_dir      ;       //  attr: sub-directory
    FLAG        marked          ;       //  entry marked YES/NO
};


SUBDIR
{
    UCHAR       path[FS_LEN]    ;       //  path name
    INT16       cur             ;       //  current entry number
    INT16       entries         ;       //  valid # of entries in array
    UINT32      total_bytes     ;       //  total bytes in all files
};


//      Prototypes
void    DIR_exit(DIRLIST *dir_e[]);
void    DIR_file_spec(STRING spec);
INT16   DIR_find(SUBDIR *dir_h,DIRLIST *dir_e[],UCHAR *fspec,INT16 attribs);
INT16   DIR_format_entry(SUBDIR *dir_h,DIRLIST *dir_e[],INT16 this_one,STRING pr_line,FLAG f_screen);
INT16   DIR_init(INT16 dir_max,SUBDIR *dir_h,DIRLIST *dir_e[]);
void    DIR_ret_filespec(UCHAR *fspec);
void    DIR_set_filespec(UCHAR *fspec);
void    DIR_show(SUBDIR *dir_h,DIRLIST *dir_e[],UCHAR *mark_msg,INT16 ercode,INT16 mark_em);
void    DIR_sort(SUBDIR *dir_h,DIRLIST *dir_e[],INT16 sort_type);
void    DIR_sort_spec(void);
INT16   FindFirstFile(UCHAR *pname,INT16 sattr,FSPEC *pfile);
INT16   FindNextFile(FSPEC *pfile);


#endif  // avoid duplicate definition
