//<t>   Header file for MNU programs

//<c>   Copyright 1980-1998 by Gerry J. Danen; all rights reserved.


#ifndef _DEFD_GD_MNU_H_
#define _DEFD_GD_MNU_H_ 1


/*      Record layouts

First line in file = copyright notice + ^Z

Record type [0] - Header Record
                  name       C  8   Menu Name
                  title      C 70   Menu Title
                  bottom_ttl C 70   Bottom Title
                  l_hdr      C 35   Left Header
                  r_hdr      C 35   Right Header
                  entries    N  2.0 Entries in Menu (max 36)
                  longest    N  2.0 Size of Longest Prompt

Record type [1] - Detail Record
                  id         C  1   Prompt Id.
                  prompt     C 70   Prompt
                  chain_menu C  8   Chain to Other Menu
                  password   C 15   Password
                  dos_pause  L  1   Pause after Execution? (Y/N)
                  dos_return L  1   Return after Execution? (Y/N)

Record type [2] - DOS Command Record (belongs to preceeding [1] record)
                  command    C 70   A single DOS command

Record type [3] - Help Record (belongs to preceeding [1] record)
                  help_line  C 70   A single line of text (may be blank)

Record type [9] - End-of-file
*/


#define MNU_X_                          struct gd_mnu_cmd_hlp_struct_
MNU_X_
{
        UCHAR   *line ;                 //  pointer to DOS or help line
        MNU_X_  *next ;                 //  pointer to next structure
};


#define MNU_D_MAX_      36
#define MNU_D_                          struct gd_mnu_det_struct_
MNU_D_
{
        UCHAR   id      ;               //  identifying item letter
        UCHAR   *prompt ;               //  prompt (max 70 chars)
        UCHAR   *chain  ;               //  next menu name (max 8 chars)
        UCHAR   *passwd ;               //  access password (max 15 chars)
        BOOL    pause   ;               //  Pause after execution? (Y/N)
        BOOL    ret     ;               //  Return after execution? (Y/N)
        MNU_X_  *cmd    ;               //  pointer to first DOS command
        MNU_X_  *hlp    ;               //  pointer to first help line
};


#define MNU_H_                          struct gd_mnu_hdr_struct_
MNU_H_
{
        UCHAR   name[9] ;               //  name of menu
        UCHAR   *title  ;               //  menu title   (max 70 chars)
        UCHAR   *b_title;               //  bottom title (max 70 chars)
        UCHAR   *l_hdr  ;               //  left header  (max 35 chars)
        UCHAR   *r_hdr  ;               //  right header (max 35 chars)
        UCHAR   d_entries ;             //  number of detail entries
        UCHAR   curr_choice ;           //  saved current choice
        MNU_D_  *det[MNU_D_MAX_] ;      //  pointer to first detail structure
        MNU_H_  *next   ;               //  pointer to next menu structure
        MNU_H_  *prev   ;               //  pointer to previous menu structure
};


#define MNU_S_                          struct gd_mnu_sys_struct_
MNU_S_
{
        UCHAR   inst_name[41] ;         //  institution name
        UCHAR   pgm_fmt  [23] ;         //  name of format program
        UCHAR   pgm_list [23] ;         //  name of list program
        UCHAR   pgm_dstat[23] ;         //  name of disk status program
        UCHAR   pgm_vtree[23] ;         //  name of visual directory tree program
        UCHAR   upd_pw   [16] ;         //  update password
        BOOL    do_colors ;             //  do colors?
};


#define MNU_STK_                        struct gd_mnu_stack_struct_
MNU_STK_
{
        UCHAR   menu_name[9] ;          //  name of menu
        UCHAR   choice ;                //  saved current choice
        MNU_STK_ *next ;                //  pointer to next structure
        MNU_STK_ *prev ;                //  pointer to previous structure
};


#define M_ITEM_                         struct gd_menu_entry_
M_ITEM_
{
        RC      row  ;                  //  row to display text
        RC      col  ;                  //  column to display text
        UCHAR   *prmt;                  //  prompt to display
};


extern  MNU_H_  *mainmenu ;             //  pointer to main menu structure
extern  MNU_H_  *curr     ;             //  pointer to current menu structure
extern  MNU_X_  *ps_hlp   ;             //  pointer to current help structure
extern  MNU_X_  *ps_cmd   ;             //  pointer to current DOS command structure


extern  FLAG    fm_scr_type    ;        //  screen type (YES=color)
extern  COL     fm_border      ;        //  border color
extern  COL     fm_fg,  fm_bg  ;        //  fg/bg colors: regular
extern  COL     fmo_fg, fmo_bg ;        //  fg/bg colors: outline
extern  COL     fmh_fg, fmh_bg ;        //  fg/bg colors: highlighted
extern  COL     fmx_fg, fmx_bg ;        //  fg/bg colors: help screen


// extern  INT16   fsm_menu_item(INT16 row,INT16 col,UCHAR id,STRING prompt);
// extern  INT16   fsm_menu_choice(INT16 initial_resp);


#endif  // avoid duplicate definition
