//<t>   Function prototypes and definitions for network functions

//<c>   Copyright 1980-1998 by Gerry J. Danen; all rights reserved.


#ifndef _DEFD_GD_NW_H_
#define _DEFD_GD_NW_H_ 1

#include"ntt.h"


#define NETWARE_REGS            union _REGS regs; struct _SREGS sregs
#define NETWARE_BINDERY         0xE3
#define CALL_NETWARE_BINDERY()  regs.h.ah=NETWARE_BINDERY; _int86x(0x21,&regs,&regs,&sregs)
#define NW_SET_BUFFER_SIZE(bufname) bufname.buffer_length=(sizeof(bufname)-2)
#if LDATA
  #define NW_DS_SI_VALUE(bufname) sregs.ds=FP_SEG(bufname);regs.x.si=FP_OFF(bufname)
  #define NW_ES_DI_VALUE(bufname) sregs.es=FP_SEG(bufname);regs.x.di=FP_OFF(bufname)
#else
  #define NW_DS_SI_VALUE(bufname) regs.x.si=(unsigned)&bufname
  #define NW_ES_DI_VALUE(bufname) regs.x.di=(unsigned)&bufname;sregs.es=sregs.ds
#endif


//      Object Types            these are hi-lo numbers...
#define UNKNOWN_OBJECT          0x0000
#define USER_OBJECT             0x0100
#define USER_GROUP_OBJECT       0x0200
#define PRINT_QUEUE_OBJECT      0x0300
#define FILE_SERVER_OBJECT      0x0400
#define JOB_SERVER_OBJECT       0x0500
#define GATEWAY_OBJECT          0x0600
#define PRINT_SERVER_OBJECT     0x0700
#define ARCHIVE_QUEUE_OBJECT    0x0800
#define ARCHIVE_SERVER_OBJECT   0x0900
#define JOB_QUEUE_OBJECT        0x0A00
#define ADMINISTRATION_OBJECT   0x0B00
#define REM_BR_SERVER_OBJECT    0x2400
#define ADV_PRINT_SERVER_OBJECT 0x4700


//      data structures
//                                      NetWare int
typedef struct  // NW_INT__
{
        UCHAR high_byte ;
        UCHAR low_byte ;
}       nw_int__ ;
typedef union   // NW_INT_
{
        nw_int__    s ;
        UINT        n ;
}       nw_int ;
//                                      NetWare long
typedef struct  // NW_LONG__
{
        UCHAR highest_byte ;
        UCHAR higher_byte ;
        UCHAR lower_byte ;
        UCHAR lowest_byte ;
}       nw_long__ ;
typedef union   // NW_LONG_
{
        nw_long__   s ;
        ULONG       n ;
}       nw_long ;
typedef struct  // NW_DATE_
{
        BYTE    y ;             // 80-99, add 1900, else add 2000
        BYTE    m ;
        BYTE    d ;
}       nw_date ;


typedef struct  OBJECT_REQUEST_
{
        WORD16  buffer_length ;
        BYTE    function ;
        nw_long prev_obj_id ;
        nw_int  obj_type ;
        BYTE    obj_name_length ;
        BYTE    obj_name[48];
}       obj_req_t ;
typedef struct  OBJECT_REPLY_
{
        WORD16  buffer_length;
        nw_long obj_id ;
        nw_int  obj_type ;
        BYTE    obj_name[48];
        BYTE    obj_flag;
        BYTE    obj_security;
        BYTE    obj_has_properties;
}       obj_rep_t ;


typedef struct  PROPERTY_REQUEST_
{
        WORD16  buffer_length ;
        BYTE    function ;
        nw_int  obj_type ;
        BYTE    obj_name_length;
        BYTE    obj_name[48];
        nw_long sequence_number ;
        BYTE    prop_name_length;
        BYTE    prop_name[16];
}       prop_req_t ;
typedef struct  PROPERTY_REPLY_
{
        WORD16  buffer_length;
        BYTE    prop_name[16];
        BYTE    prop_flags;
        BYTE    prop_security;
        nw_long sequence_number ;
        BYTE    prop_has_value;
        BYTE    more_properties;
}       prop_rep_t ;


typedef struct  VALUE_REQUEST_
{
        WORD16  buffer_length ;
        BYTE    function ;
        nw_int  obj_type ;
        BYTE    obj_name_length;
        BYTE    obj_name[48];
        BYTE    segment_number;
        BYTE    prop_name_length;
        BYTE    prop_name[16];
}       val_req_t ;
typedef struct  VALUE_REPLY_
{
        WORD16  buffer_length;
        ULONG   prop_value[32];
        BYTE    more_segments;
        BYTE    prop_flags;
}       val_rep_t ;


typedef struct  GBON_REQUEST_
{
        WORD16  buffer_length ;
        BYTE    function ;
        nw_long obj_id ;
}       gbon_req_t ;
typedef struct  GBON_REPLY_
{
        WORD16  buffer_length ;
        nw_long obj_id ;
        nw_int  obj_type ;
        BYTE    obj_name[48] ;
}       gbon_rep_t ;


typedef struct  GCI_REQUEST_                    // Get Connection Information
{
        WORD16  buffer_length ;
        BYTE    function ;
        BYTE    conn_number ;
}       gci_req_t ;
typedef struct  GCI_REPLY_
{
        WORD16  buffer_length ;
        ULONG   obj_id ;
        WORD16  obj_type ;
        BYTE    obj_name [48] ;
        BYTE    login_info[8] ;
}       gci_rep_t ;


typedef struct  // LOGIN_CONTROL__
{
        nw_date ac_expires ;                    // 3    1-3
        BYTE    acct_disabled ;                 // 1    4
        nw_date pw_expires ;                    // 3    5-7
        BYTE    grace_left ;                    // 1    8
        nw_int  days_betw_pw_changes ;          // 2    9-10
        BYTE    grace_allowed ;                 // 1    11
        BYTE    min_pw_length ;                 // 1    12
        nw_int  sim_connect   ;                 // 2
        BYTE    login_time_bitmap[42] ;         // 42
        nw_date last_login ;                    // 3    57-59  last login date
        BYTE    last_login_time[3] ;            // 3
        BYTE    restrictions ;                  // 1
        BYTE    unused     ;                    // 1
        nw_long max_disk_blocks ;               // 4
        nw_int  bad_login_count ;               // 2
        nw_long next_reset_time ;               // 4
        BYTE    bad_login_address[14] ;         // 14
        BYTE    filler00[40] ;                  // 40 x '00'
}       login_control_t ;
/*
        Format of the LOGIN_CONTROL Bindery Property
        --------------------------------------------

Offset  Field                           Size Type       Comment

0       Account Expiration Date         3    bytes      Yr Mn Dy
3       Account Disabled Flag           1    byte       00h or FFh
4       Password Expiration Date        3    bytes      Yr Mn Dy
7       Grace Logins Remaining          1    byte       FFh=unlimited
8       Password Expiration Interval    2    word       Days
10      Grace Login Reset Value         1    byte
11      Minimum Password Length         1    byte
12      Maximum Concurrent Connections  2    word
14      Allowed Login Time Bitmap       42   bytes      1 bit per 1/2 hr
56      Last Login Date and Time        6    bytes      Yr Mn Dy Hr Mi Sc
62      Restriction Flags               1    byte
63      Unused                          1    byte
64      Maximum Disk Usage in Blocks    4    long       Block=4k
68      Bad Login Count                 2    word
70      Next Reset Time                 4    long
74      Bad Login Address               14   bytes      net:node:socket
*/


//      Prototypes
// INT32   nw_bindery_date_to_ymd(nw_date *in);
// int     nw_call_w_buffers(int function_call,void *request_buffer,void *reply_buffer);
// int     nw_deallo_dir_handle(BYTE dirHandle);
// void    nw_del_drive_mapping(UCHAR drv);
// void    nw_del_fakeroot(UCHAR drv);
// BOOL    nw_fileServerData(STRING osVersion,STRING osRevision,STRING osCopyright,STRING osSerialNumber,STRING osAppNumber,int *maxUsers);
// int     nw_get_connection_number(void);
// int     nw_get_conn_info(int conn,BYTE objName[],WORD16 *objType,long *objID,INT32 *loginDate,INT32 *loginTime,BYTE inetNetwork[],BYTE inetNode[],BYTE inetSocket[]);
// int     nw_get_default_connection_id(void);
// BYTE    nw_get_dir_handle(UCHAR drv);
// BYTE FAR * nw_get_fserver_name_table(void);
// void    nw_get_station_address(BYTE node[]);
// ULONG   nw_get_username(STRING username);
// BOOL    nw_installed_NetWare(void);
// BOOL    nw_installed_PC_LAN(void);
// BOOL    nw_is_supervisor(STRING u);
// BOOL    nw_is_user_in_group(STRING uName,STRING gName);
// BOOL    nw_logmsg(STRING instuff);
// ULONG   nw_long_hilo_to_lohi(ULONG number);
// int     nw_map_fakeroot(UCHAR drv,STRING path_to_map);
// int     nw_maximum_users(void);
// STRING  nw_server_name(WORD16 ConnectionID);
// WORD16  nw_word_hilo_to_lohi(WORD16 number);


// extern  BOOL    netware_installed_ ;

//      nw_get_connection_info is replaced by nw_get_conn_info
// ULONG   nw_get_connection_info(int conn,WORD16 *obj_type,STRING obj_name,INT32 *login_date,INT32 *login_time);


INT32   nw_bindery_date_to_ymd(nw_date *in);
int     nw_call_w_buffers(int function_call,void *request_buffer,void *reply_buffer);
int     nw_deallo_dir_handle(BYTE dirHandle);
void    nw_del_drive_mapping(UCHAR drv);
void    nw_del_fakeroot(UCHAR drv);
BOOL    nw_fileServerData(STRING osVersion,STRING osRevision,STRING osCopyright,STRING osSerialNumber,STRING osAppNumber,int *maxUsers);
int     nw_get_connection_number(void);
int     nw_get_conn_info(int conn,BYTE objName[],WORD16 *objType,long *objID,INT32 *loginDate,INT32 *loginTime,BYTE inetNetwork[],BYTE inetNode[],BYTE inetSocket[]);
int     nw_get_default_connection_id(void);
BYTE    nw_get_dir_handle(UCHAR drv);
BYTE FAR * nw_get_fserver_name_table(void);
void    nw_get_station_address(BYTE node[]);
ULONG   nw_get_username(STRING username);
BOOL    nw_installed_NetWare(void);
BOOL    nw_installed_PC_LAN(void);
BOOL    nw_is_supervisor(STRING u);
BOOL    nw_is_user_in_group(STRING uName,STRING gName);
BOOL    nw_logmsg(STRING instuff);
ULONG   nw_long_hilo_to_lohi(ULONG number);
int     nw_map_fakeroot(UCHAR drv,STRING path_to_map);
int     nw_maximum_users(void);
STRING  nw_server_name(WORD16 ConnectionID);
int     nw_verify_password_of_object(STRING oName,STRING oPass,WORD16 oType);
WORD16  nw_word_hilo_to_lohi(WORD16 number);
int     nw_get_obj_restrictions(WORD serverConnID,BYTE volNumber,LONG objectID,LONG *volRestriction,LONG *blocksInUse,BOOL *isRestricted);
int     nw_send_msg_to_group(STRING gname,UCHAR msg[]);
int     nw_send_msg_to_user(STRING uname,UCHAR msg[]);
STRING  nw_formatted_bindery_date(nw_date *in,STRING out);
int     nw_shell_version(int *majorV,int *minorV,int *revision);
STRING  nw_ws_os_type(STRING sData);
STRING  nw_ws_os_ver(STRING sData);
STRING  nw_ws_hw_type(STRING sData);
STRING  nw_ws_hw_stype(STRING sData);
int     nw_drive_mapping(BYTE drv,STRING path);
BOOL    nw_login_drive_letter(char *drLetter,STRING targetFile,UCHAR driveLetters[]);


#endif  // avoid duplicate definition
